@@ -466,7 +466,7 @@ class ClUpdateAction(Action):
          },
          {'name': 'update:set_current_level',
          'method': 'Update.update_increment_current_level()',
-         'depend': Tasks.success(), 
+         'depend': (Tasks.success() & Tasks.hasnot("interrupt") & Tasks.success_all("update")), 
          'condition': lambda Get: Get('cl_update_sync_only_set') == 'off' and
                 Get('cl_update_pretend_set') == 'off' and
                 Get('update.cl_update_use_migration_host') == 'on'
