@@ -1888,10 +1888,35 @@ class VariableClBinhostStash(Variable):
     """
     value = ""
 
-class VariableClCheckUpdateLevel(Variable):
+class VariableClUpdateIgnoreLevel(Variable):
     """
     Проверять уровень обновления 
     при соединении с binhost
     """
     type = "bool"
-    value = "on"
\ No newline at end of file
+    opt = ["--ignore-level"]
+    value = "off"
+
+    def init(self):
+        self.label = _("Forcefully ignore update level and migration system")
+        self.help = _("forcefully ignore update level and migration system")
+
+class VariableClUpdateForceLevel(Variable):
+    """
+    Принудительное обновление до определенного уровня
+    """
+    type = "int"
+    opt = ["--force-level"]
+    metavalue = "LEVEL"
+
+
+    def init(self):
+        self.label = _("Force update to particular level")
+        self.help = _("force update to particular level")
+
+class VariableClUpdateForceLevelMode(Variable):
+    """
+    Режим принудительного обновления до определенного уровня
+    """
+    type = "bool"
+    value = "off"
